#include "SourceRegions.h"
#include <errstr.h>
#include "CalServer.h"


//#include "param.h"
//#include "ParamInfoSet.h"
//#include "Region.h"

//#include <cmath>
//#include <algorithm>
//#include <limits>

using namespace std;

SourceRegions::SourceRegions(std::string spec_file)
: _spec_file(spec_file)
{
	readRegions();
}

void SourceRegions::readRegions()
{
	string file = _spec_file;
	DataSet * reg_set = dataSetServer->open(file,Dal::Read);
	if (!reg_set->hasBlock("REGION"))
		errstr << "No REGION extension in the Region file. Exiting."<< Sas::error("WrongRegionFile");

	if (reg_set->hasAttribute("OBS_ID") )
				_obs_id = reg_set->attribute("OBS_ID")->asString();
	if (reg_set->hasAttribute("CONTENT") )
				_content = reg_set->attribute("CONTENT")->asString();
	if (reg_set->hasAttribute("EXPIDSTR") )
				_expid = reg_set->attribute("EXPIDSTR")->asString();


	if (reg_set->hasBlock("SPECTRUM")){
	Table * spec_tab = reg_set->table("SPECTRUM");					// Read information from the Spectrum
	if (spec_tab->hasAttribute("SRCNUM") )
					_srcnum = spec_tab->attribute("SRCNUM")->asInt();
	if (spec_tab->hasAttribute("BACKSCAL") )
					_backscale = spec_tab->attribute("BACKSCAL")->asInt();

	if (spec_tab->hasAttribute("SRC_RA") )
					_src_ra_dec.push_back(spec_tab->attribute("SRC_RA")->asReal());
	if (spec_tab->hasAttribute("SRC_DEC") )
					_src_ra_dec.push_back(spec_tab->attribute("SRC_DEC")->asReal());

	unsigned spec_N = spec_tab->rows();
	ColumnData *tab_spec_counts = spec_tab->column("COUNTS")->data();
	int32 *spec_counts = tab_spec_counts->int32Data();
				_total_spec_counts = 0.0;
				for (unsigned long i = 0 ; i < spec_N ; ++i){
					_total_spec_counts += spec_counts[i];
				}
				//cout << "-------> Spec file		 : " << _spec_file <<endl;
				//cout << "		_total_spec_counts : " << _total_spec_counts <<endl;
	} else {
		errstr << "No SPECTRUM extension in the Region file. Brightness cuts will be estimated from the image. "
				"Expecting to find Source number "
				"\"SRCNUM\" and coord. \"SRC_RA, SRC_DEC\" in extension 1 ..."<< Sas::message(Msg::App, Msg::Sparse);
		Table * spec_tab_ = reg_set->table(0);					// Read information from Extension 0
		if (spec_tab_->hasAttribute("SRCNUM") ){
							_srcnum = spec_tab_->attribute("SRCNUM")->asInt();
							//cout << "-------> SRCNUM		 : " << _srcnum <<endl;
					} else {
						errstr << "No SPECTRUM extension in the Region file and Source number \"SRCNUM\" not found in Extension 1."
										<< Sas::error("WrongRegionFile");
					}
		if (spec_tab_->hasAttribute("SRC_RA") )
							_src_ra_dec.push_back(spec_tab_->attribute("SRC_RA")->asReal());
		if (spec_tab_->hasAttribute("SRC_DEC") )
							_src_ra_dec.push_back(spec_tab_->attribute("SRC_DEC")->asReal());
		//cout << "-------> RA, Dec		 : " << _src_ra_dec[0] << ", " << _src_ra_dec[1] << endl;
		_total_spec_counts = 0.0;
	}


	Table * reg_tab = reg_set->table("REGION");						// Read information from the REGION
	int nshape, nx, ny, nr;
	if (reg_tab -> hasColumn("SHAPE"))
		nshape = reg_tab -> columnNumber("SHAPE");
	if (reg_tab -> hasColumn("X"))
		nx = reg_tab -> columnNumber("X");
	if (reg_tab -> hasColumn("Y"))
		ny = reg_tab -> columnNumber("Y");
	if (reg_tab -> hasColumn("R"))
		nr = reg_tab -> columnNumber("R");

	//string mtype1, mform1;
	if (reg_tab->hasAttribute("MTYPE1") )					// pos	or	eqpos
		_mtype1 = reg_tab->attribute("MTYPE1")->asString();
	if (reg_tab->hasAttribute("MFORM1") )					// X,Y	or	RA---TAN, DEC--TAN
		_mform1 = reg_tab->attribute("MFORM1")->asString();

			//cout << "			nshape, nx, ny, nr : " << nshape << ", " << nx << ", " << ny << ", " << nr << endl;
			//cout << "			mtype1, mform1 : " << mtype1 << ", " << mform1 << endl;
			if (nshape != 0 || nx != 1 || ny != 2 || nr != 3)
				errstr << "REGION extension does not contain valid column names (or the correct order): SHAPE, X, Y, R. Exiting."<< Sas::error("WrongRegionFile");

	unsigned reg_N = reg_tab->rows();
	ColumnData *tab_reg_shape = reg_tab->column("SHAPE")->data();
	//char *reg_shape = tab_reg_shape->stringData();
	ColumnData *tab_reg_x = reg_tab->column("X")->data();
	real32 *reg_X = tab_reg_x->real32Data();
	ColumnData *tab_reg_y = reg_tab->column("Y")->data();
	real32 *reg_Y = tab_reg_y->real32Data();
	ColumnData *tab_reg_r = reg_tab->column("R")->data();
	real32 *reg_R = tab_reg_r->real32Data();

	// SHAPE of the REGION 1st row:
	char *shape_1strow = tab_reg_shape -> stringData() + tab_reg_shape -> position(0);
	_shape_1strow = shape_1strow;

	unsigned elementNr = 0;
	//string first_shape;
	for (unsigned long i = 0 ; i < reg_N ; ++i){
		vector<double> to_insert;
		char *shape = tab_reg_shape -> stringData() + tab_reg_shape -> position(elementNr);
		string shape_ = shape;
		  to_insert.push_back(reg_X[i]);
		  to_insert.push_back(reg_Y[i]);

		  if (shape_ == "ANNULUS" && i == 0){
		  		to_insert.push_back(reg_R[0]);
		  		to_insert.push_back(reg_R[1]);
		  		//first_shape = "ANNULUS";
		  	}
		  if (_shape_1strow == "ANNULUS" && shape_ == "!CIRCLE" && i > 0) {
		  		to_insert.push_back(reg_R[2*i]);
		  	}

		  if (_shape_1strow == "CIRCLE" && (shape_ == "CIRCLE" || shape_ == "!CIRCLE")) {
		  		to_insert.push_back(reg_R[i]);
		  	}

		_regions.insert(_regions.end(), to_insert.begin(), to_insert.end());
	elementNr++;
	//cout << "			i, reg_R[i], reg_R[2*i] : " << i << ", " << reg_R[i] << ", " << reg_R[2*i] << endl;
	}
	//cout << endl;
		//int size = _regions.size();
		//cout << "-------> _regions size : " << size <<endl;
		//for (int i = 0 ; i < size ; ++i){
		//	cout << "			regions i, data : " << i << ", " << _regions[i] << endl;
		//}

		//cout << "			regions size, data : " << size << ", " << _regions[0] << ", " << _regions[1] << ", " << _regions[2] << endl;
		//if (size > 3){
		//	cout << "			regions size, data : " << size << ", " << _regions[3] << ", " << _regions[4] << ", " << _regions[5] << endl;
		//	cout << "			regions size, data : " << size << ", " << _regions[6] << ", " << _regions[7] << ", " << _regions[8] << endl;
		//}
		//cout << endl;

	//delete reg_tab;
	dataSetServer->close(reg_set);

}


double SourceRegions::getTotalSpecCounts()
{
	return _total_spec_counts;
}

std::vector<double> SourceRegions::getSrcRADec()
{
	return _src_ra_dec;
}

std::vector<double> SourceRegions::getRegion()
{
	//return _res;
	return _regions;
}

string SourceRegions::getobs_id()
{
	return _obs_id;
}

string SourceRegions::getcontent()
{
	return _content;
}

string SourceRegions::getMType1()
{
	return _mtype1;
}

string SourceRegions::getMForm1()
{
	return _mform1;
}

int SourceRegions::getsrcnum()
{
	return _srcnum;
}

int SourceRegions::getbackscale()
{
	return _backscale;
}

string SourceRegions::getShape1stRow()
{
	return _shape_1strow;
}

string SourceRegions::getExpid()
{
	return _expid;
}
