/*
 implotregions

   Authors:  Jose Vicente Perea - XMM-Newton SOC
         
   Description: Source and background extraction regions in EPIC images
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/

/*
 implotregions

   Authors:  Jose Vicente Perea - XMM-Newton SOC

   Description: EPIC source and background extraction regions thumbnails

   Comments:

  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/


#include "implotregions.h"

#include <Tracer.h>
#include <errstr.h>
#include "param.h"
#include <cmath>

#include "CalServer.h"
#include "Dal.h"

#include "loadImage.h"
#include "plotImage.h"
#include "SourceRegions.h"

////////////////////////////////
//  To drawImage
//
#include "cpgplot.h"
#include <cmath>
#include <iostream>
#include <vector>
#include <algorithm>
#include <numeric>
////////////////////////////////

using namespace std;

void implotregions ()
{
PROC

/* Thumbnails of EPIC images showing source and background extraction regions. */
/* (Plots EPIC images using PGPLOT and WCS) */

/* Get the parameters from this command line */

    // Read input options:
	//
	std::string imagefile = stringParameter("imageset");	// Image file
	std::string spec_file = stringParameter("srcregfile");	// Source spectrum file
	std::string bkg_file = stringParameter("bkgregfile");	// Background spectrum file
    //bool withZClip = booleanParameter("withzclip");		// Manual image cuts. Fixed. Not configurable.
    bool withXYClip = booleanParameter("withxyclip");		// Cropped image
    double separation = realParameter("separationmargin");	// Separation between the display box and the extraction regions.
    std::string outputFile = stringParameter("outputfile");	// Output PNG thumbnail file.

    //=============================================================================================
    //
    // Read regions from REGION extensions in spectra
    //string spec_file = "/home/jperea/XMM/SAS/data/extract_regions_thumbnails/spectra/P0741030601PNS003SRSPEC0002.FTZ";
    //string bkg_file = "/home/jperea/XMM/SAS/data/extract_regions_thumbnails/spectra/P0741030601PNS003BGSPEC0002.FTZ";
    SourceRegions regionsrc(spec_file);
    SourceRegions regionbkg(bkg_file);
    string obs_id_src = regionsrc.getobs_id();
    string obs_id_bkg = regionbkg.getobs_id();
    string expid_src = regionsrc.getExpid();
    string expid_bkg = regionbkg.getExpid();
    string contentsrc = regionsrc.getcontent();
    string contentbkg = regionbkg.getcontent();
    string mtype1src = regionsrc.getMType1();
    string mtype1bkg = regionbkg.getMType1();
    string mform1src = regionsrc.getMForm1();
    string mform1bkg = regionbkg.getMForm1();
    int src_num_src = regionsrc.getsrcnum();

    //cout << "		mtype1src, mtype1bkg		: " << mtype1src << " = " << mtype1bkg  << endl;
    if (mtype1src != mtype1bkg)
    	errstr << "Source and Background regions are in different coordinate type. Src: " << mtype1src << " , Bkg: " << mtype1bkg
		<< " . Both must be in the same coord. type."<< Sas::error("WrongRegionFile");
    string regcoordtype = mtype1src;
    if (regcoordtype != "pos")			// Not completed for Regions in RA, Dec ('eqpos'). TBD
        errstr << "Sorry, at present the task will only deal with MTYPE1 = \"pos\", CTYPE = \"X/Y\"."<< Sas::error("badWcsType");


    vector<double> src_ra_dec = regionsrc.getSrcRADec();
    	// START - SRC_RA and SRC_DEC from deg to HMS,DMS //////////////////////
    		double ra = src_ra_dec[0];
    		double dec = src_ra_dec[1];
    		string rs, ds;
    		double raH, raM, raS, decD, decM, decS;
    		double raH_, raM_, decD_, decM_;
    		// RA
    		if (ra < 0)
    			rs="-"; else rs="+";
    			raH_ = modf(ra/15, &raH);
    			raM_ = modf(raH_ * 60.0, &raM);
    			raS = raM_ * 60.0;
    		// Dec
    		if (dec < 0)
    			ds="-"; else ds="+";
    			decD_ = abs(modf(dec, &decD));
    			decD = abs(decD);
    			decM_ = abs(modf(decD_ * 60.0, &decM));
    			decM = abs(decM);
    			decS = abs(decM_ * 60.0);

    		// RA
    			stringstream raH_s;
    			raH_s << raH;
    			string raH_s_;
    			if (raH < 10){raH_s_ = "0" + raH_s.str();} else {raH_s_ = raH_s.str();}
    			//const char * raHstr = raH_s_.c_str();

    			stringstream raM_s;
    			raM_s << raM;
    			string raM_s_ = raM_s.str();
    			if (raM < 10){raM_s_ = "0" + raM_s.str();} else {raM_s_ = raM_s.str();}
    			//const char * raMstr = raM_s_.c_str();

    			stringstream raS_s;
    			raS_s << raS;
    			string raS_s_ = raS_s.str();
    			//const char * raSstr = raS_s_.c_str();
    		// Dec
    			stringstream decD_s;
    			decD_s << decD;
    			string decD_s_;
    			if (decD < 10){decD_s_ = ds + "0" + decD_s.str();} else {decD_s_ = ds + decD_s.str();}
    			//const char * decDstr = decD_s_.c_str();

    			stringstream decM_s;
    			decM_s << decM;
    			string decM_s_ = decM_s.str();
    			if (decM < 10){decM_s_ = "0" + decM_s.str();} else {decM_s_ = decM_s.str();}
    			//const char * decMstr = decM_s_.c_str();

    			stringstream decS_s;
    			decS_s << decS;
    			string decS_s_ = decS_s.str();
    			//const char * decSstr = decS_s_.c_str();
    			//cout << endl;
    			//cout << "		src_ra_dec[0]		: " << ra << " = " << rs << raH << ":" << raM <<":" << raS << endl;
    			//cout << "		src_ra_dec[1]		: " << dec << " = " << ds << decD << ":" << decM <<":" << decS << endl;
    			//cout << endl;
    	// END - SRC_RA and SRC_DEC from deg to HMS,DMS //////////////////////

    int src_num_bkg = regionbkg.getsrcnum();
    int backscale_bkg = regionbkg.getbackscale();
    double total_spec_counts = regionbkg.getTotalSpecCounts();
    string shape_bkg_1strow = regionbkg.getShape1stRow();

    vector<double> src_region = regionsrc.getRegion();
    vector<double> bkg_region = regionbkg.getRegion();
    	    cout << endl;
    	    cout << "	OBS_ID			: " << obs_id_src << endl;
    	    cout << "		SRCNUM		: " << src_num_src << endl;
    	    //cout << "		CONTENT			: " << contentsrc << " ; " << contentbkg << endl;
    	    //cout << "		BACKSCAL bkg		: " << backscale_bkg << endl;
    	    //cout << "		RA, Dec		: " << src_ra_dec[0] << ", " << src_ra_dec[1] << endl;
    	    cout << "		RA---TAN	: " << raH_s_ << ":" << raM_s_ <<":" << raS_s_ << endl;
    	    cout << "		DEC--TAN	: " << decD_s_ << ":" << decM_s_ <<":" << decS_s_ << endl;
    	    cout << "		Background shape: " << shape_bkg_1strow << endl;
    	    cout << endl;
    	    //cout << "		Src region (X,Y,R) (0.05 arcsec)	: " << src_region[0] << ", " << src_region[1] << ", " << src_region[2] << endl;
    	    //cout << "		shape_bkg_1strow			: " << shape_bkg_1strow << endl;
    	    //if (shape_bkg_1strow == "ANNULUS"){
    	    //	cout << "		Bkg region (X,Y,R) (0.05 arcsec)	: " << bkg_region[0] << ", " << bkg_region[1] << ", " << bkg_region[2] << ", " << bkg_region[3]<< endl;
    	    //} else {
    	    //	cout << "		Bkg region (X,Y,R) (0.05 arcsec)	: " << bkg_region[0] << ", " << bkg_region[1] << ", " << bkg_region[2] << endl;
    	    //}

        double world[2];
            world[0] = src_region[0];
            world[1] = src_region[1];
        double r     = src_region[2];
            //cout << "		world (X,Y,r) (0.05 arcsec)		: " << world[0] << ", " << world[1] << ", "<< r << endl;
        double world_bkg[2];
            world_bkg[0] = bkg_region[0];
            world_bkg[1] = bkg_region[1];
        double r_bkg     = bkg_region[2];
            //cout << "		world_bkg (X,Y,r) (0.05 arcsec)		: " << world_bkg[0] << ", " << world_bkg[1]<< ", " << r_bkg << endl;
            //cout << endl;

    vector<double> src_and_bkg_regions;							// All Regions included : Src + Bkg + Contaminatng sources included in Bkg spectrum
    		src_and_bkg_regions.push_back(src_region[0]);
    		src_and_bkg_regions.push_back(src_region[1]);
    		src_and_bkg_regions.push_back(src_region[2]);
    		//cout << "		bkg_region size		: " << bkg_region.size() << endl;
    		for (unsigned long i = 0 ; i < bkg_region.size() ; ++i){
    			src_and_bkg_regions.push_back(bkg_region[i]);
    		}
    //=============================================================================================


    // Read input image:
	//
	//loadImage image_(imagefile, withZClip, shape_bkg_1strow, src_and_bkg_regions, backscale_bkg, total_spec_counts);
	loadImage image_(imagefile, regcoordtype, shape_bkg_1strow, src_and_bkg_regions, backscale_bkg, total_spec_counts);
	//
    float * image = image_.getImage();
	vector<int> size = image_.getImageSize();
	vector<double> effMinMax = image_.getImageeffMinMax();

	// Image information from Keywords
	string obs_id = image_.getobs_id();
	string instrument = image_.getInstr();
	string datamode = image_.getDatamode();
	string submode = image_.getSubmode();
	string expid = image_.getExpid();
	string filter = image_.getFilter();
	string object = image_.getObject();
	string observer = image_.getObserver();
	string date_obs = image_.getDate();
	double exposure = image_.getExp();
	CoordPair PixelSizeWorld = image_.getPixelSizeWorld();
	if (PixelSizeWorld.x() != PixelSizeWorld.y())		// Both == 80
		errstr << "xPixelSizeWorld and yPixelSizeWorld differ. Exiting."<< Sas::error("WrongPixelSizeWorld");

	//cout << "		OBS_ID				: " << obs_id << endl;
	//cout << "		Image size: xsize, ysize	: " << size[0] << ", " << size[1] << endl;
    //cout << "		effMin ( min(image) ), effMax ( max(image) ) : " << effMinMax[0] << ", " << effMinMax[1] << endl;
    //cout << "		Exposure IDs				: " << expid << ", "<< expid_src << ", "<< expid_bkg << endl;

    //if (obs_id != obs_id_src)
    //	errstr << "Image and Source Region file do not correspond to the same observation. Exiting."<< Sas::error("WrongRegionFile");
    //if (obs_id != obs_id_bkg)
    //    errstr << "Image and Background Region file do not correspond to the same observation. Exiting."<< Sas::error("WrongRegionFile");
    //if (expid != expid_src)
    //    errstr << "Image and Source Region file do not correspond to the same exposure. Exiting."<< Sas::error("WrongRegionFile");
    //if (expid != expid_bkg)
    //    errstr << "Image and Background Region file do not correspond to the same exposure. Exiting."<< Sas::error("WrongRegionFile");
    if (src_num_src != src_num_bkg)
        errstr << "Source number of the Source and Background spectra differ. Exiting."<< Sas::error("WrongSourceNumbers");

    //if (contentsrc != "EPIC SOURCE SPECTRUM")
    //    errstr << "Source Region file is not a proper EPIC spectrum. CONTENT keyword != \"EPIC SOURCE SPECTRUM\". Exiting."<< Sas::error("WrongRegionFile");
    //if (contentbkg != "EPIC SOURCE BACKGROUND SPECTRUM")
    //    errstr << "Background Region file is not a proper EPIC spectrum. CONTENT keyword != \"EPIC SOURCE BACKGROUND SPECTRUM\". Exiting."<< Sas::error("WrongRegionFile");

    // Plot image
    //
    //
    // Open plot device:
    stringstream srcnum;
    srcnum << setw(4) << setfill('0') << src_num_src;
    //string outputpng = "png_images/thumb_" + obs_id + "_" + instrument + "_" + expid + "_SRC" + srcnum.str() + ".png/PNG";				// TBD
    //string outputpng = outputFile + "thumb_" + obs_id + "_" + instrument + "_" + expid + "_SRC" + srcnum.str() + ".png/PNG";
    //
    string outputpng = outputFile + "/PNG";
    // outputFile = input parameter at command line : "outputfile"
    //
    int ok =  cpgbeg(0, outputpng.c_str(), 1, 1);
    		    if (!ok) {
    		    	errstr << "Plot device not created. Exiting."<< Sas::error("badPlotDevice");
    		    }

    // Whole image or clipped per source+background pair
    //
    vector<int> xy_limits;

    if (shape_bkg_1strow == "ANNULUS" || !withXYClip) {				// shape_bkg_1strow == "ANNULUS" or NO XYClipping
    	xy_limits = image_.getXY_limits();

    } else if ( shape_bkg_1strow == "CIRCLE"){
            // Clip image around the Source and Background regions
    		//
            vector<double> x_cricle_borders;
            vector<double> y_cricle_borders;
            	x_cricle_borders.push_back(world[0]-r);
            	x_cricle_borders.push_back(world[0]+r);
            	x_cricle_borders.push_back(world_bkg[0]-r_bkg);
            	x_cricle_borders.push_back(world_bkg[0]+r_bkg);
            	y_cricle_borders.push_back(world[1]-r);
            	y_cricle_borders.push_back(world[1]+r);
            	y_cricle_borders.push_back(world_bkg[1]-r_bkg);
            	y_cricle_borders.push_back(world_bkg[1]+r_bkg);
            std::vector<double>::const_iterator minx_ =  std::min_element(x_cricle_borders.begin(), x_cricle_borders.end());
            std::vector<double>::const_iterator maxx_ =  std::max_element(x_cricle_borders.begin(), x_cricle_borders.end());
            std::vector<double>::const_iterator miny_ =  std::min_element(y_cricle_borders.begin(), y_cricle_borders.end());
            std::vector<double>::const_iterator maxy_ =  std::max_element(y_cricle_borders.begin(), y_cricle_borders.end());
            double Dx = abs(*maxx_ - *minx_);
            double Dy = abs(*maxy_ - *miny_);
            double D = max(Dx, Dy);
            double x_center = *minx_ + Dx/2;
            double y_center = *miny_ + Dy/2;
            //cout << "		DEBUG for Margins Dx, Dy, D : " << Dx << ", "<< Dy << " --> "<< D << endl;
            //cout << "		DEBUG for Margins r_bkg : " << r_bkg << endl;

            // Reference for separation = Ref_sep
            // Border separation = fraction of the reference = frac * (Ref_sep)
            //   double Ref_sepx = r_bkg;
            //   double Ref_sepy = r_bkg;
            //   double Ref_sepx = Dx;
            //   double Ref_sepy = Dy;
            double Ref_sep = D;

            //int minx = (int)((*minx_-separation*Ref_sep)/PixelSizeWorld.x());		// Image margins from the Src+Bkg regions borders
            int minx = (int)((x_center-separation*Ref_sep)/PixelSizeWorld.x());		// Image margins from the center of the Src+Bkg pair
            	if (minx < 1)
            		minx = 1;
            //int maxx = (int)((*maxx_+separation*Ref_sep)/PixelSizeWorld.x());
            int maxx = (int)((x_center+separation*Ref_sep)/PixelSizeWorld.x());
            	if (maxx > size[0])
            		maxx = size[0];
            //int miny = (int)((*miny_-separation*Ref_sep)/PixelSizeWorld.x());
            int miny = (int)((y_center-separation*Ref_sep)/PixelSizeWorld.x());
            	if (miny < 1)
            		miny = 1;
            //int maxy = (int)((*maxy_+separation*Ref_sep)/PixelSizeWorld.x());
            int maxy = (int)((y_center+separation*Ref_sep)/PixelSizeWorld.x());
        		if (maxy > size[1])
        			maxy = size[1];

        	//double x_box_size = abs(maxx - minx);
        	//double y_box_size = abs(maxy - miny);

            xy_limits.push_back(minx);
            xy_limits.push_back(maxx);
            xy_limits.push_back(miny);
            xy_limits.push_back(maxy);
    }

    //cout << "		minx, maxx, miny, maxy			: " << xy_limits[0] << ", " << xy_limits[1] << " ; "
	//										     	 	<< xy_limits[2] << ", " << xy_limits[3] << endl;

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    plotImage(image, effMinMax, size, xy_limits, imagefile, instrument, datamode, submode, filter, object, observer, date_obs, exposure, spec_file);

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    // Include Source and Background regions
    //

    // Read WCS info from image input file
    //
    string ctype1 = image_.getxType();
    string ctype2 = image_.getyType();
    //CoordPair wcsRefWorld = image_.getRefCoords();
    //CoordPair wcsRefPixels = image_.getRefPixels();
    //CoordPair PixelSizeWorld = image_.getPixelSizeWorld();
    string cunit1 = image_.getXUnit();
    string cunit2 = image_.getYUnit();
    //real64 rotAngle = image_.getrotAngle();
    std::vector<real64> MatrixOnOffAxis = image_.getMatrixOnOffAxis();

    // getPixVec : All regions from Src and Bkg spectra ( src_and_bkg_regions ) converted to pixel coordinates:
    //
    vector<double> pixVec_ = image_.getPixVec();

    //cout << endl;
    //cout << "		CTYPE1/2		: " << ctype1 << ", "<< ctype2 << endl;
    //cout << "		CUNIT/2			: " << cunit1 << ", "<< cunit2 << endl;
    //cout << "		wcs[x,y]RefWorld	CRVAL	: " << wcsRefWorld.x() << ", "<< wcsRefWorld.y() << endl;
    //cout << "		wcs[x,y]RefPixels	CRPIX	: " << wcsRefPixels.x() << ", "<< wcsRefPixels.y() << endl;
    //cout << "		[x,y]PixelSizeWorld	CDELT	: " << PixelSizeWorld.x() << ", "<< PixelSizeWorld.y() << endl;
    //cout << "		rotAngle		: " << rotAngle << endl;
    //cout << endl;

    if (ctype1 != "X" or ctype2 != "Y"){
    	errstr << "Sorry, at present the task will only deal with CTYPE = X/Y."<< Sas::error("badWcsType");
    }
    //if (ctype1 != "RA---TAN" or ctype2 != "DEC--TAN"){
    //    	errstr << "Sorry, at present the task will only deal with CTYPE = RA/DEC--TAN."<< Sas::error("badWcsType");
    //}
    if (cunit1 != "deg" or cunit2 != "deg"){
     	errstr << "Sorry, at present only degrees are supported."<< Sas::error("badWcsWorldUnit");
    }


    int colsave;
    int textClrIndex = 1;				// text Colour index : intParameter('srccolour'). Def = 7. Green = 3
    int textClrIndex_bkg = 1;
    int textClrIndex_cont = 2;
    int fontsave;
    int thicksave;
    int lineThicknessPixels = 3;
    float csize;
    float textSize = 1.0;
    cpgqci(&colsave);					// inquire color index
    cpgsci(textClrIndex);				// text Colour index : intParameter('srccolour'). Def = 7
    cpgqcf(&fontsave);					// inquire character font
    cpgscf(1);							// set character font
    cpgsfs(2);							// set fill-area style. For hollow circles
    cpgqlw(&thicksave);					// inquire line width
    cpgslw(lineThicknessPixels);		// intParameter('thickness'). Def = 1
    cpgqch(&csize);						// inquire character height
    cpgsch(textSize);					// realParameter('labelsize'). Def = 1


    float x_, y_, x_src, y_src, rad_, rad1_, rad2_;
    for (unsigned long i = 0 ; i < (pixVec_.size()/2) ; ++i){
        	//cout << "		pixVec		: " << pixVec_[0+(i*2)] << ", " << pixVec_[1+(i*2)] << endl;
            x_ = pixVec_[0+(i*2)] + 0.5;
            y_ = pixVec_[1+(i*2)] + 0.5;

            if ( shape_bkg_1strow == "CIRCLE" || shape_bkg_1strow == "!CIRCLE"){
            	rad_ = src_and_bkg_regions[2+(i*3)]/PixelSizeWorld.x();
            } else if ( shape_bkg_1strow == "ANNULUS" ){
            	if (i == 0){
            		rad_ = src_and_bkg_regions[2+(i*3)]/PixelSizeWorld.x();}
            	if (i == 1){	// ANNULUS
            		rad1_ = src_and_bkg_regions[0+2+(i*3)]/PixelSizeWorld.x();
            		rad2_ = src_and_bkg_regions[1+2+(i*3)]/PixelSizeWorld.x();}
            	if (i > 1){		// Contaminating circles
                        rad_ = src_and_bkg_regions[1+2+(i*3)]/PixelSizeWorld.x();}
            }

        	//cout << "		x,y,rad	NEW	: " << x_ << ", " << y_ << ", " << rad_ << endl;

            if (i==0){												// Source circle
            	cpgsls(1);
            	cpgsci(textClrIndex);
        		cpgcirc(x_, y_, rad_);
        		stringstream srcn;
        	    srcn  << src_num_src;
        	    string label = srcn.str();
        	    const char * label_ = label.c_str();
        	    cpgptxt(x_, y_ + 1.1 * rad_, 0.0, 0.0, label_);
        	    //cout << "		x,y,rad	: " << x_ << ", " << y_ << ", " << rad_ << endl;
        	    x_src = x_;
        	    y_src = y_;
        	} else if (i==1){										// Background circle
        		cpgsls(2);
        		cpgsci(textClrIndex_bkg);
        		if ( shape_bkg_1strow == "CIRCLE" ){
        		cpgcirc(x_, y_, rad_);
        		} else if ( shape_bkg_1strow == "ANNULUS" ){
        			cpgcirc(x_, y_, rad1_);
        			cpgcirc(x_, y_, rad2_);
        		}
        	} else {
        		cpgsls(1);
        		cpgsci(textClrIndex_cont);							// Contaminating source circles
        		cpgcirc(x_, y_, rad_);
        	}
            cpgsls(1);
        	//cout << "		x,y,rad	: " << x_ << ", " << y_ << ", " << rad_ << endl;
        }



    // Source center coordinates at bottom ///////////////////
    //
    cpgsci(1);
    const char * ra_dec = ("Source center RA: " + raH_s_ + " " + raM_s_ +" "+raS_s_+" Dec: "+ decD_s_ + " " + decM_s_ + " " + decS_s_).c_str();
    cpgmtxt("B", 5.0, 0.5, 0.5, ra_dec);
    /////////////////////////////////////////////////////////

    //
    // Box labels
    //
    cpgsci(1);
    cpgscf(2);			// better quality Roman font
    //cpgswin(0.0, 1.0, 0.0, 1.0);
    double x_0 = xy_limits[0] - x_src;
    double x_1 = xy_limits[1] - x_src;
    double y_0 = xy_limits[2] - y_src;
    double y_1 = xy_limits[3] - y_src;

    ////cpgswin(xy_limits[0]*80/20/60, xy_limits[1]*80/20/60, xy_limits[2]*80/20/60, xy_limits[3]*80/20/60);	// Box limits = minx, maxx, miny, maxy
    cpgswin(x_0*PixelSizeWorld.x()/20/60, x_1*PixelSizeWorld.x()/20/60, y_0*PixelSizeWorld.x()/20/60, y_1*PixelSizeWorld.x()/20/60);											// Box limits = Centered in source
	cpgbox("BCINTS", 0.0, 0, "BCINTS", 0.0, 0);
    cpglab("X (arcmin)", "Y (arcmin)", "");
    cpgwedg("LI", 4.9, 2.3, effMinMax[0], effMinMax[1], " ");
    cpgsch(0.6);
    cpgmtxt("T", 1.2, -0.11, 1.0, "cts/pixel");
    //cpgsch(1.0);

    // Close plot device
    cpgclos();
    //
    //cout << "		DEBUG -----  " <<  endl;


ENDPROC
}
