#include "plotImage.h"
#include <errstr.h>
#include "cpgplot.h"
//#include "StringToken.h"
#include <StringFns.h>

//#include <cmath>
//#include <algorithm>
//#include <limits>
using namespace std;

plotImage::plotImage(float* image, std::vector<double> effMinMax, std::vector<int> size, std::vector<int> xy_limits, std::string imagefile, std::string instr, std::string datamode, std::string submode, std::string filter, std::string object, std::string observer, std::string date_obs, double exposure, std::string spec_file)
: _image(image), _effMinMax(effMinMax), _size(size), _xy_limits(xy_limits), _imagefile(imagefile),  _instr(instr), _datamode(datamode), _submode(submode), _filter(filter), _object(object), _observer(observer), _date_obs(date_obs), _exposure(exposure), _spec_file(spec_file)
{
	calPlot();
}

void plotImage::calPlot()
{
	_minx = _xy_limits[0];
	_maxx = _xy_limits[1];
	_miny = _xy_limits[2];
	_maxy = _xy_limits[3];
	//int JUST = 1;		// JUST=1, the scales of the x and y axes (in world coordinates per inch) will be equal)
	//int AXIS = 0;
	//cpgenv(minx, maxx, miny, maxy, JUST, AXIS);

	// call pgsvp(rhVpXMin, rhVpXMax, rhVpYMin, rhVpYMax)
	float mainVpXMin = 0.19;
	float mainVpXMax = 0.81;
	float mainVpYMin = 0.156;
	float mainVpYMax = 0.95;
	float mainCharSize = 0.7;
	float rhVpXMin = mainVpXMax;
	float rhVpXMax = 1.0;
	float rhVpYMin = 0.206;
	float rhVpYMax = 1.0;
	_rhCharSize = 0.5;
	_rhCharX = 0.05;
	_rhCharDy = 0.05;
	_rhCharXindent = 0.04;
	_rhCharYindent = -0.02;
	//float botVpXMin = 0.1;	// location info for BottomInfo at bottom left side (optional for the future)
	//float botVpXMax = 1.0;
	//float botVpYMin = 0.0;
	//float botVpYMax = 0.1;
	//float botCharSize = 0.7;
	//float botCharX = 0.0;
	//float botCharDy = 0.21;
	cpgsvp(rhVpXMin, rhVpXMax, rhVpYMin, rhVpYMax);

	// Difference of magnitude between effective cuts
	double ratio = _effMinMax[1] / _effMinMax[0];
	int imageTransferId;
		if (ratio < 10.0){
			imageTransferId = 0;
			_imageTransferStyle = "linear";
		} else if (ratio >= 10.0 && ratio < 1000.0) {
			imageTransferId = 2;
			_imageTransferStyle = "sqrt";
		} else if (1000.0 <= ratio) {
			imageTransferId = 1;
			_imageTransferStyle = "log";
		}
	//cout << "		_effMinMax, RATIO, imageTransferStyle   : " << _effMinMax[0] << ", " <<_effMinMax[1]<< ", " << ratio << ", " << _imageTransferStyle << endl;
	cpgsitf(imageTransferId);			// 0=linear (def) 1=log 2=sqrt

	// plotRHinfo(minx, maxx, miny, maxy, imageSet, imageArray)
	plotRHinfo();

	// pgsvp(mainVpXMin, mainVpXMax, mainVpYMin, mainVpYMax)
	cpgsvp(mainVpXMin, mainVpXMax, mainVpYMin, mainVpYMax);

	cpgwnad(_minx-0.5, _maxx+0.5, _miny-0.5, _maxy+0.5);

	//float mainCharSize = 0.7;
	cpgsch(mainCharSize);

	set_palette();		// Set colours palette


	//float * trimage = TransformMatrix( minx, dx, miny, dy);
	float * trimage = TransformMatrix();

	cpgimag( _image, _size[0], _size[1], _minx, _maxx, _miny, _maxy, _effMinMax[0], _effMinMax[1], trimage );
	errstr<<"Highest image value shown: " << _effMinMax[0] <<Sas::message(Msg::App, Msg::Verbose);
	errstr<<"Lowest image value shown: " << _effMinMax[1] <<Sas::message(Msg::App, Msg::Verbose);

	std::list<std::string> imagefile_abs = StringFns::splitString(_imagefile, "/");
	std::string imagefile = imagefile_abs.back();
	const char * title = imagefile.c_str();
	cpgmtxt("T", 1.4, 0.5, 0.5, title);

	//cpgscf(2);			// better quality Roman font

	// gridStyle = stringParameter('gridstyle'). Def.: 'ticks'
	// std::string gridStyle = "ticks";
	//if(gridStyle != "none"){
	//	const char * lab = "BC";
	//	const char * empt = " ";
	//	const char * wcsYctype = "wcs%Y%ctype";
	//	cpgbox(lab, 0.0, 0, lab, 0.0, 0);
	//  cpglab(empt, wcsYctype, empt);
	//} else {
		//const char * lab = 'BCINT';
		//const char * X = 'X';
		//const char * Y = 'Y';
		//const char * empt = ' ';

	//	//cpgswin(0.0, 1.0, 0.0, 1.0);
	//	cpgbox("BCINTS", 0.0, 0, "BCINTS", 0.0, 0);
	//  cpglab("X", "Y", "");

	//}

//	if(gridStyle != 'none'){
		//idents(1) = wcs%x%ctype
		//idents(2) = " "
		//idents(3) = " "
		//ic        = -1
		//dparam(4)     = wcs%x%refPixels
		//dparam(5)     = wcs%y%refPixels
		//dparam(6:9)   = (/1.0d0, 0.0d0, 0.0d0, 1.0d0/)

//	}


	cpgiden();


}


// plotRHinfo(minx, maxx, miny, maxy, imageSet, imageArray)
void plotImage::plotRHinfo()
{
	////cpgswin(0.0, 1.0, 0.0, 1.0);
	float x = _rhCharX;
	float y = 0.98;
	float dy = _rhCharDy;
	float xoffset = _rhCharXindent;
	float yoffset = _rhCharYindent;

	int font;
	float csize;
	cpgqcf(&font);
	cpgscf(1);
	cpgqch(&csize);

    // Info::Task current task info
	std::string taskName = Info::Task::current()->name();
	std::string taskVer = Info::Task::current()->version();
	std::string task = taskName + "-" + taskVer;
    cpgsch(0.7);
	const char * legend = task.c_str();
    cpgtext(x, y, legend);

    y = y - 2.0 * dy;

    cpgsch(_rhCharSize);

    const char * instr = "Instrument:";
    cpgtext(x, y, instr);
    const char * string = _instr.c_str();
    cpgtext(x+xoffset, y+yoffset, string);
    y = y - 0.5 * dy;
    cpgtext(x+xoffset, y+yoffset, (_datamode + ", " + _submode).c_str());

    y = y - dy;
    const char * filter = "Filter:";
    cpgtext(x, y, filter);
    cpgtext(x+xoffset, y+yoffset, _filter.c_str());

    y = y - dy;
    const char * object = "Object";
    cpgtext(x, y, object);
    cpgtext(x+xoffset, y+yoffset, _object.c_str());

    y = y - dy;
    const char * observer = "Observer:";
    cpgtext(x, y, observer);
    cpgtext(x+xoffset, y+yoffset, _observer.c_str());

    y = y - dy;
    const char * date_obs = "DATE-OBS:";
    cpgtext(x, y, date_obs);
    cpgtext(x+xoffset, y+yoffset, _date_obs.c_str());

    y = y - dy;
    const char * exp = "Exposure:";
    cpgtext(x, y, exp);

    std::ostringstream strs;
    strs << _exposure;
    std::string str = strs.str();
    const char * expo = (str + " sec").c_str();
    cpgtext(x+xoffset, y+yoffset, expo);

    y = y - 1.5 * dy;
    cpgtext(x, y, "Image size:");
    std::ostringstream minx, maxx, miny, maxy;
    minx << _minx;
    maxx << _maxx;
    miny << _miny;
    maxy << _maxy;
    const char * img_size = ("pixels (" + minx.str() + ":" + maxx.str() + ", " + miny.str() + ":" + maxy.str() + ")").c_str();
    cpgtext(x+xoffset, y+yoffset, img_size);

    y = y - 1.5 * dy;
    cpgtext(x, y, "Parameters:");
    y = y - 0.5 * dy;
    const char * scaletype = ("scaletype = " + _imageTransferStyle).c_str();
    cpgtext(x+xoffset, y, scaletype);

    y = y - 0.5 * dy;
    cpgtext(x+xoffset, y, "Min/Max brightness");
    y = y - 0.5 * dy;
    cpgtext(x+2*xoffset, y, "truncation levels:");

    y = y - 0.5 * dy;
    std::ostringstream strs_1, strs_2;
    strs_1 << _effMinMax[0];
    strs_2 << _effMinMax[1];
    std::string str_1 = strs_1.str();
    std::string str_2 = strs_2.str();
    const char * effMinMax = (str_1 + " / " + str_2).c_str();
    cpgtext(x+6*xoffset, y, effMinMax);

    // Spectrum file name:
    y = y - 1.0 * dy;
    cpgtext(x, y, "Spectrum filename:");
    y = y - 0.5 * dy;
    std::list<std::string> specfile_abs = StringFns::splitString(_spec_file, "/");
    std::string spec_file = specfile_abs.back();
    const char * spec = spec_file.c_str();
    cpgtext(x-xoffset, y, spec);
    //cout << "		Spec file		: " << spec << endl;

    y = y - 2 * dy;
    cpgtext(x+xoffset, y, "White solid circle:");
    y = y - 0.5 * dy;
    cpgtext(x+4*xoffset, y, "Source extraction region.");

    y = y - 1 * dy;
    cpgtext(x+xoffset, y, "White dashed circle:");
    y = y - 0.5 * dy;
    cpgtext(x+4*xoffset, y, "Background extraction region.");

    y = y - 1 * dy;
    cpgtext(x+xoffset, y, "Red solid circles:");
    y = y - 0.5 * dy;
    cpgtext(x+4*xoffset, y, "Excluded regions.");

}


void plotImage::set_palette()
{
    float *cl = new float[7];
    	cl[0] = 0.1;
    	cl[1] = 0.25;
    	cl[2] = 0.43;
    	cl[3] = 0.57;
    	cl[4] = 0.71;
    	cl[5] = 0.8;
    	cl[6] = 1.0;

    float *cr = new float[7];
   		cr[0] = 0.5;
    	cr[1] = 0.0;
    	cr[2] = 0.0;
    	cr[3] = 0.0;
    	cr[4] = 0.9;
    	cr[5] = 1.0;
    	cr[6] = 1.0;
    float *cg = new float[7];
		cg[0] = 0.0;
		cg[1] = 0.0;
		cg[2] = 0.8;
		cg[3] = 1.0;
		cg[4] = 0.9;
		cg[5] = 0.3;
		cg[6] = 1.0;
    float *cb = new float[7];
		cb[0] = 0.5;
		cb[1] = 1.0;
		cb[2] = 0.8;
		cb[3] = 0.0;
		cb[4] = 0.0;
		cb[5] = 0.3;
		cb[6] = 1.0;
	cpgctab(cl, cr, cg, cb, 8, 1.0, 0.5);
	cpgscr(16, 0.0, 0.0, 0.0);
}

/**************************************************
     * Subroutine TransformMatrix
     * Sets up plotting transformation matrix.

****************************************************/
//float* TransformMatrix(float xmin, float xbinWidth, float ymin, float ybinWidth)
float* plotImage::TransformMatrix()
    {
        float *trimage = new float[6];

       //trimage[0] = xmin - xbinWidth / 2.0;
        trimage[0] = 0.0;
        trimage[1] = 1.0;

        //trimage[1] = xbinWidth;
        trimage[2] = 0.0;
        trimage[3] = 0.0;
        //trimage[3] = ymin - ybinWidth / 2.0;
        trimage[4] = 0.0;
        //trimage[5] = ybinWidth;
        trimage[5] = 1.0;

        return trimage;
    }



//std::vector<double> drawImage::getDraw()
//{
//	return _res;
//}
